﻿using Proto;
using Proto.Cluster;

namespace TestProject;

/// <summary>
/// 스마트 전구 그레인
/// </summary>
public class SmartBulbGrain : SmartBulbGrainBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Enumeration
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 스마트 전구 상태 - SmartBulbState

    /// <summary>
    /// 스마트 전구 상태
    /// </summary>
    private enum SmartBulbState { Unknown, On, Off }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 클러스터 ID
    /// </summary>
    private readonly ClusterIdentity clusterIdentity;

    /// <summary>
    /// 스마트 전구 상태
    /// </summary>
    private SmartBulbState state = SmartBulbState.Unknown;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - SmartBulbGrain(context, clusterIdentity)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="context">컨텍스트</param>
    /// <param name="clusterIdentity">클러스터 ID</param>
    public SmartBulbGrain(IContext context, ClusterIdentity clusterIdentity) : base(context)
    {
        this.clusterIdentity = clusterIdentity;

        Console.WriteLine($"생성 : {this.clusterIdentity.Identity}");
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 켜기 - TurnOn()

    /// <summary>
    /// 켜기
    /// </summary>
    /// <returns>태스크</returns>
    public override async Task TurnOn()
    {
        if(this.state != SmartBulbState.On)
        {
            Console.WriteLine($"스마트 전구 켜기 : {this.clusterIdentity.Identity}");

            this.state = SmartBulbState.On;
        }
    }

    #endregion
    #region 끄기 - TurnOff()

    /// <summary>
    /// 끄기
    /// </summary>
    /// <returns>태스크</returns>
    public override async Task TurnOff()
    {
        if(this.state != SmartBulbState.Off)
        {
            Console.WriteLine($"스마트 전구 끄기 : {this.clusterIdentity.Identity}");

            this.state = SmartBulbState.Off;
        }
    }

    #endregion
    #region 상태 구하기 - GetState()

    /// <summary>
    /// 상태 구하기
    /// </summary>
    /// <returns>스마트 전구 상태 구하기 결과</returns>
    public override Task<GetSmartBulbStateResponse> GetState()
    {
        return Task.FromResult(new GetSmartBulbStateResponse { State = this.state.ToString() });
    }

    #endregion
}