﻿using Proto;
using Proto.Cluster;

namespace TestProject;

/// <summary>
/// 스마트 전구 시뮬레이터
/// </summary>
public class SmartBulbSimulator : BackgroundService
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 액터 시스템
    /// </summary>
    private readonly ActorSystem actorSystem;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - SmartBulbSimulator(actorSystem)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="actorSystem">액터 시스템</param>
    public SmartBulbSimulator(ActorSystem actorSystem)
    {
        this.actorSystem = actorSystem;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 실행하기 (비동기) - ExecuteAsync(cancellationToken)

    /// <summary>
    /// 실행하기 (비동기)
    /// </summary>
    /// <param name="cancellationToken">취소 토큰</param>
    /// <returns>태스크</returns>
    protected override async Task ExecuteAsync(CancellationToken cancellationToken)
    {
        Random random = new Random(DateTime.Now.Millisecond);

        string[] lightBulbArray = new string[]
        {
            "거실1",
            "거실2",
            "침실",
            "부엌"
        };

        while(!cancellationToken.IsCancellationRequested)
        {
            string id = lightBulbArray[random.Next(lightBulbArray.Length)];

            SmartBulbGrainClient client = this.actorSystem
                .Cluster()
                .GetSmartBulbGrain(id);

            if(random.Next(2) > 0)
            {
                await client.TurnOn(cancellationToken);
            }
            else
            {
                await client.TurnOff(cancellationToken);
            }

            await Task.Delay(TimeSpan.FromMilliseconds(500), cancellationToken);
        }
    }

    #endregion
}